#!/bin/bash
# ***************************************************************************
# *                         WARRANTY DISCLAIMER
# ***************************************************************************
# * THE SAMPLE SCRIPT IS BEING PROVIDED TO YOU "AS IS".  DELL DISCLAIMS ANY
# * AND ALL WARRANTIES, EXPRESS, IMPLIED OR STATUTORY, WITH RESPECT TO THE
# * SAMPLE SCRIPT, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND ANY WARRANTY
# * OF NON-INFRINGEMENT. YOU WILL USE THE SAMPLE SCRIPT AT YOUR OWN RISK.
# * DELL SHALL NOT BE LIABLE TO YOU FOR ANY DIRECT OR INDIRECT DAMAGES
# * INCURRED IN USING THE SAMPLE SCRIPT. IN NO EVENT SHALL DELL OR ITS
# * SUPPLIERS BE RESPONSIBLE FOR ANY DIRECT OR INDIRECT DAMAGES WHATSOEVER
# * (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF PROFITS, LOSS OF USE,
# * LOSS OF DATA, BUSINESS INTERRUPTION, OR OTHER PECUNIARY LOSS, NOR FOR
# * PUNITIVE, INCIDENTAL, CONSEQUENTIAL, OR SPECIAL DAMAGES OF ANY KIND,
# * UNDER ANY PART OF THIS AGREEMENT, EVEN IF ADVISED OR AWARE OF THE
# * POSSIBILITY OF SUCH DAMAGES).
# ***************************************************************************

# ***************************************************************************
# * Script Name: lininst.sh
# *
# * Purpose:
# *   This script performs an unattended installation of Red Hat Linux.
# *   Supported Linux OSs are:
# *     Red Hat Enterprise Linux 3, Red Hat Enterprise Linux 4 and 
# *     Red Hat Enterprise Linux 5.
# *     
# * Layout: 
# *   This script is divided into the following sections for ease of use and
# *   extensibility:
# *     Section 1: Set Paths for Deployment Toolkit Tools and Scripts
# *     Section 2: Ask User's Input, Input Verification, and Code Execution
# *     Section 3: Script Outcome (Success or Failure and Error Handling) 
# *
# * Tools Used: 
# *   grub is used as the bootloader to load the vmlinuz and initrd.img
# *   from a deployment partition.
# *   Dell provided KS.CFG with default setup components.
# *
# * External Dependencies
# *   DT_TOOLS, DT_SYSTEMS set by tkenvset.sh let task scripts
# *   know where they can find Toolkit Tools and Configuration scripts.
# * Note:
# * This script assumes that there exists only one controller with vdisks.
# * 
# ***************************************************************************
# ***************************************************************************
# * Function to display the outcome (Success or Failure and Error Handling)
# * Expects $? as an argument to display the Return/Error Code.
# **************************************************************************
toolfail ()
{
	echo racadm failed with error code $1.
	errhndl.sh $0 32
	exit
}

# ***************************************************************************
# * filerrror: Function to display an error if any required file was not found.
# ***************************************************************************
fileerror ()
{
	echo $0: A file needed to install Linux was not found. Aborting.
	exit
}

# ***************************************************************************
# * Section 1: Set The environment variables if required.
# ***************************************************************************

if [ "$DT_TOOLS"="" ]; then
	source tkenvset.sh
fi

#This environment variable is the disk Eg /dev/sda /dev/hda on which to deploy linux.
if [ -z $DT_HD ]; then
	echo environment variable DT_HD is not set, exiting.
	exit 1;
fi
#This environment variable is the location of the initrd.img vmlinuz and ks.cfg.
if [ -z $DT_OS_SRC ]; then
	echo environment variable DT_OS_SRC is not set, exiting.
	exit 1;
fi

# Ensure that partcfg.sh file is executed before this. It will create the
# OS Deployment partition for this installation to use. 

# Check for pending values, if exists ask user to reboot.
test=""
idrac_type=""
test=`syscfg --idractype 2> /dev/null`
idrac_type=$test
if [ $idrac_type -ge 16 ];
then
	pstatus=""
	source pendingvaluecheck.sh
	if [ $pstatus -eq 0 ]
	then
		echo Reboot to apply pending values. Exiting...
		exit 1
	fi	
fi

#bootmode check
syscfg --bootmode > /dev/null 2>&1
bmvalid=$?

boot_mode=""
if [ $bmvalid -eq 0 ]
then
	boot_mode=`syscfg --bootmode 2> /dev/null`
fi

#11G specific bios->uefi
if [ "$boot_mode" = "bootmode=uefi" ]
then
	efibootmgr 2&> /dev/null
	if [ $? -ne 0 ]
	then
		echo Reboot to apply bootmode to UEFI mode. Exiting...
		exit 1
	fi
fi

#11G specific uefi->bios
if [ "$boot_mode" = "bootmode=bios" ]
then
	efibootmgr 2&> /dev/null
	if [ $? -eq 0 ]
	then
		echo Reboot to apply bootmode to BIOS mode. Exiting...
		exit 1
	fi
fi

# Check for bootmode and choose partition schemes accordingly
if [ $bmvalid -ne 0 ] || [ "$boot_mode" = "bootmode=bios" ]
then
	# Check to see if we created the utility partition in a previous step.
	GRUB_PART_NO=1
	if ( fdisk -l $DT_HD | grep "Dell Utility" > /dev/null 2>&1 ); then
		# Dell Utility partition found.
		# install grub in the boot partition
		GRUB_PART_NO=1
		GRUB_DISK=hd0,1
		DT_OS_PART=${DT_HD}2
	else
		# Dell Utility partition not found.
		GRUB_PART_NO=0
		GRUB_DISK=hd0,0
		DT_OS_PART=${DT_HD}1
	fi

	# The location where grub staging files are located.
	GRUB_STAGE_FILES_LOCATION=/usr/share/grub/*/
	# The mount point for the primary partition
	if [ ! -e /tmp/dp ]; then
		mkdir /tmp/dp
	fi

	DT_MOUNT=/tmp/dp

	if ( ! mount $DT_OS_PART $DT_MOUNT ); then
		echo "Mount failed."
		exit
	fi

	if [ ! -e $DT_OS_SRC/initrd.img ] || [ ! -e $DT_OS_SRC/vmlinuz ] \
	 || [ ! -e $DT_OS_SRC/ks.cfg ]; then
		fileerror
	fi

	mkdir -p $DT_MOUNT/linux
	cp $DT_OS_SRC/initrd.img $DT_MOUNT/linux
	cp $DT_OS_SRC/vmlinuz $DT_MOUNT/linux
	cp $DT_OS_SRC/ks.cfg $DT_MOUNT/linux/ks.cfg

	# Copy over grub staging files
	mkdir -p $DT_MOUNT/boot/grub
	cp $GRUB_STAGE_FILES_LOCATION/* $DT_MOUNT/boot/grub/

	#Create a devicemap file to force /dev/sda to be (hd0)
> $DT_MOUNT/boot/grub/device.map
cat >> $DT_MOUNT/boot/grub/device.map << _EOF
(hd0) $DT_HD
_EOF

	#install grub into boot sector of partition.
grub --device-map=$DT_MOUNT/boot/grub/device.map << _EOF
root (hd0,$GRUB_PART_NO)
setup ($GRUB_DISK)
_EOF

	if [ $? -ne 0 ]; then
		echo Grub was not installed on the target drive
		exit 1
	fi

	#Create grub.conf
	#Set the location of ks.cfg hd:sda? to indicate that
	#ks.cfg is located on sda1 or sda2 on the hard disk.
	#kernel /linux/vmlinuz ks=hd:sda$(($GRUB_PART_NO + 1))/linux/ks.cfg
> $DT_MOUNT/boot/grub/grub.conf
cat >> $DT_MOUNT/boot/grub/grub.conf << _EOF
default=0
timeout=10
title Dell Deployment Toolkit Red Hat install
	root (hd0,$GRUB_PART_NO)
	kernel /linux/vmlinuz ks=bd:80p$(($GRUB_PART_NO + 1)):/linux/ks.cfg
	initrd /linux/initrd.img
_EOF

else
	echo Deploying OS in UEFI mode
	GRUB_PART_NO=1
	DT_OS_PART=${DT_HD}${GRUB_PART_NO}

	# The mount point for the primary partition
	if [ ! -e /tmp/dp ]; then
		mkdir /tmp/dp
	fi
	DT_MOUNT=/tmp/dp

	if ( ! mount $DT_OS_PART $DT_MOUNT ); then
		echo "Mount failed."
		exit
	fi
	
	if [ ! -e $DT_OS_SRC/initrd.img ] || [ ! -e $DT_OS_SRC/vmlinuz ] || [ ! -e $DT_OS_SRC/ks_uefi.cfg ]; then
		fileerror
	fi

	#copy vmlinuz, initrd and ks_uefi.cfg into deployment partition
	mkdir -p $DT_MOUNT/linux
	cp $DT_OS_SRC/initrd.img $DT_MOUNT/linux
	cp $DT_OS_SRC/vmlinuz $DT_MOUNT/linux
	cp $DT_OS_SRC/ks_uefi.cfg $DT_MOUNT/linux/ks_uefi.cfg
	
	# The location where grub2 efi binaries are located.
	GRUB2_EFI_LOCATION=/opt/dell/grub2

	# Copy grub2 binaries
	mkdir -p $DT_MOUNT/efi/grub
	cp $GRUB2_EFI_LOCATION/grub.efi $DT_MOUNT/efi/grub/

	#Create grub.cfg
> $DT_MOUNT/efi/grub/grub.cfg
cat >> $DT_MOUNT/efi/grub/grub.cfg << _EOF
default=0
timeout=10
set menu_color_normal=white/black
set menu_color_highlight=black/light-gray
menuentry "RHEL INSTALLATION (UEFI MODE) THROUGH DTK" {
	set gfxpayload=keep
	linux	/linux/vmlinuz ks=hd:sda${GRUB_PART_NO}/linux/ks_uefi.cfg
	initrd	/linux/initrd.img
}
_EOF
	
	# Create a entry in uefi boot manager
	efibootmgr --create --gpt --disk $DT_HD --part ${GRUB_PART_NO} --write-signature --label "RHEL_DTK_UEFI" --loader "\\EFI\\grub\\grub.efi"

fi

umount $DT_MOUNT

echo Rebooting the system.
if [ $bmvalid -ne 0 ] || [ "$boot_mode" = "bootmode=bios" ]
then
	echo 'Setting bootseq for bios mode...'
	test=""
	test=`syscfg --idractype 2> /dev/null`
	idractype=$test
	if [ $idractype -ge 16 ];
	then
		syscfg --bootseq=HardDisk.List.1-1
	else
		syscfg --bootseq=hdd.emb.0,hdd.emb.1
	fi
fi


echo "Waiting for 20 seconds to REBOOT... or Ctrl C to abort."
count=20
while true; do
	sleep 1
	count=$(($count-1))
	if [ $count -eq 0 ]; then
		reboot
	fi
	echo -e "Counting $count \r\c"
done

# The following line should not be reached.
echo done.
