#!/bin/bash

# ***************************************************************************
# *                         WARRANTY DISCLAIMER
# ***************************************************************************
# * THE SAMPLE SCRIPT IS BEING PROVIDED TO YOU "AS IS".  DELL DISCLAIMS ANY
# * AND ALL WARRANTIES, EXPRESS, IMPLIED OR STATUTORY, WITH RESPECT TO THE
# * SAMPLE SCRIPT, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND ANY WARRANTY
# * OF NON-INFRINGEMENT. YOU WILL USE THE SAMPLE SCRIPT AT YOUR OWN RISK.
# * DELL SHALL NOT BE LIABLE TO YOU FOR ANY DIRECT OR INDIRECT DAMAGES
# * INCURRED IN USING THE SAMPLE SCRIPT. IN NO EVENT SHALL DELL OR ITS
# * SUPPLIERS BE RESPONSIBLE FOR ANY DIRECT OR INDIRECT DAMAGES WHATSOEVER
# * (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF PROFITS, LOSS OF USE,
# * LOSS OF DATA, BUSINESS INTERRUPTION, OR OTHER PECUNIARY LOSS, NOR FOR
# * PUNITIVE, INCIDENTAL, CONSEQUENTIAL, OR SPECIAL DAMAGES OF ANY KIND,
# * UNDER ANY PART OF THIS AGREEMENT, EVEN IF ADVISED OR AWARE OF THE
# * POSSIBILITY OF SUCH DAMAGES).
# ***************************************************************************

# ***************************************************************************
# * Script Name: partcfg.sh
# *
# * Purpose:
# *   This script creates and populates the Dell Utility Partition as well as
# *   creates the Deployment Partition on a specified disk.
# *
# * Layout: 
# *   This script is divided into the following sections for ease of use and
# *   extensibility:
# *     Section 1: Set Paths for Deployment Toolkit Tools and Scripts
# *     Section 2: Ask User's Input, Input Verification, and Code Execution
# *
# * Tools Used: 
# *   fdisk to create the primary partition for OS
# *   upinit.sh to create the Dell Utility Partition
# *	
# *
# * External Dependencies
# *   DT_TOOLS, DT_SYSTEMS set by tkenvset.sh let task scripts
# *   know where they can find Toolkit Tools and Configuration scripts.
# *   DT_PLATFORM is set by tkenvset.sh using Platform Discovery
# ***************************************************************************

# ***************************************************************************
# * Function to display the outcome (Success or Failure and Error Handling)
# * Expects $? as an argument to display the Return/Error Code.
# ***************************************************************************
toolfail ()
{
	echo partcfg failed with error code $1.
	$DT_SCRIPTS/errhndl.sh $0 32
	# Exit the script if Return Value is 2
	if [ $? -eq 2 ]; then
		echo "Exiting."
		exit 2
	fi
}

# ***************************************************************************
# createprimpart ()
# Creates a Primary partition using fdisk utility
#
# Takes two arguments
# ===================
# $1 is the Primary Partition Number (1-4)
# $2 is the Size (in MB) of the Partition
# 
# External Dependencies
# DT_TEMP_FILE should to be set to a writable file in /tmp.
# DT_HD should be the harddisk on which the partition needs to be created.
# ***************************************************************************
createprimpart ()
{
	if [ -z "$1" ] || [ -z "$2" ]; then
		echo "Invalid parameters to createprimepart"
		exit 1
	fi

	echo -e "n\np\n$1\n\n+${2}M\nw\n" > $DT_TEMP_FILE
	fdisk $DT_HD < $DT_TEMP_FILE > /dev/null 2>&1
	udevadm settle
	sync
}

# ***************************************************************************
# Function to create the deployment partition.
# This partition will be used to store the files required to kickstart a
# RedHat Linux Installation.
# ***************************************************************************
createdp ()
{
	# Create the deployment partition
	createprimpart $DT_PART_NUM $DT_DP_SIZE
	
	# Set the bootable flag
	echo -e "a\n$DT_PART_NUM\nw\n" > $DT_TEMP_FILE
	fdisk $DT_HD < $DT_TEMP_FILE
	udevadm settle
	sync
	# Format the deployment partition
	mkfs.ext2 "$DT_HD"$DT_PART_NUM
	retval=$?
	if [ $retval -ne 0 ]; then
		toolfail $retval
	fi
}

# ***************************************************************************
# Function to create the gpt deployment partition (EFI System Partition).
# This partition will be used to store the files required to kickstart a
# RHEL/SLES installation in UEFI mode.
# 
# External Dependencies
# DT_TEMP_FILE should to be set to a writable file in /tmp.
# DT_HD should be the harddisk on which the partition needs to be created.
# ***************************************************************************
createdp_gpt()
{
	# check for validity of input
	if [ -z "$DT_PART_NUM" ] || [ -z "$DT_DP_SIZE_GPT" ]; then
		echo "Invalid parameter to createdp_gpt"
		exit 1
	fi

	echo Creating ESP...
	echo -e "mkpart primary fat32 1mib $DT_DP_SIZE_GPT mib\nset 1 boot on\nquit\n" > $DT_TEMP_FILE
	parted $DT_HD < $DT_TEMP_FILE > /dev/null 2>&1
	udevadm settle
	sync

	echo Formating partition to vfat 32 type... 
	mkfs.vfat -F 32 "$DT_HD"$DT_PART_NUM
	retval=$?
	if [ $retval -ne 0 ]; then
		toolfail $retval
	fi	
}
	
	
# ***************************************************************************
# * Section 1: Set Paths for Deployment Toolkit Tools and Scripts
# ***************************************************************************
if [ -z $DT_TOOLS ]; then
	. tkenvset.sh
fi

# #############################################################################
#                          USER MODIFICATION REQUIRED
# Below are the user defined variables that this script uses.
# In a full deployment, these variables should be customized to fit the 
# user's needs.
# #############################################################################
# Assign default values to variables if they are not assigned elsewhere.

# Hard disk on which to create the Dell Utility and deployment partitions.
[ -z "$DT_HD" ] && DT_HD=/dev/sda

# Size of the Dell Utility Partition
[ -z "$DT_UP_SIZE" ] &&	DT_UP_SIZE=32

# Utility Partition Image
[ -z "$DT_UP_IMAGE" ] && DT_UP_IMAGE=$DT_SYSTEMS/upimg.bin

# Enable/disable creating the new Deployment Partition
[ -z "$DT_PARTN_DEP" ] && DT_PARTN_DEP=ON

# Size of the Deployment Partition in megabytes
[ -z "$DT_DP_SIZE" ] &&	DT_DP_SIZE=64

# Size of the GPT Deployment Partition in mib's 
[ -z "$DT_DP_SIZE_GPT" ] && DT_DP_SIZE_GPT=125
# ***************************************************************************
# * Section 2: Ask User's Input, Input Verification, and Code Execution
# ***************************************************************************
# Primary partition number - used when creating the partitions using fdisk
DT_PART_NUM=1

# Temporary file used to pass arguments to fdisk
DT_TEMP_FILE=`mktemp /tmp/dtktemp.XXXXXX`

# Check for pending values, if exists ask user to reboot.
test=""
idrac_type=""
test=`syscfg --idractype 2> /dev/null`
idrac_type=$test
if [ $idrac_type -ge 16 ];
then
	pstatus=""
	source pendingvaluecheck.sh
	if [ $pstatus -eq 0 ]
	then
		echo Reboot to apply pending values. Exiting...
		exit 1
	fi	
fi

#bootmode check
syscfg --bootmode > /dev/null 2>&1
bmvalid=$?

boot_mode=""
if [ $bmvalid -eq 0 ]
then
	boot_mode=`syscfg --bootmode 2> /dev/null`
fi

#11G specific bios->uefi 	
if [ "$boot_mode" = "bootmode=uefi" ]
then
	efibootmgr 2&> /dev/null
	if [ $? -ne 0 ]
	then
		echo Reboot to apply bootmode to UEFI mode. Exiting...
		exit 1
	fi
fi

#11G specific uefi->bios
if [ "$boot_mode" = "bootmode=bios" ]
then
	efibootmgr 2&> /dev/null
	if [ $? -eq 0 ]
	then
		echo Reboot to apply bootmode to BIOS mode. Exiting...
		exit 1
	fi
fi

# Check for bootmode and choose partition schemes accordingly
if [ $bmvalid -ne 0 ] || [ "$boot_mode" = "bootmode=bios" ]
then
	echo 'Creating MBR (BIOS mode) partitions...'

	# Enable/disable creating the new Dell Utility Partition
	[ -z "$DT_PARTN_UP" ] && DT_PARTN_UP=ON

	# ***************************************************************************
	# Clear all partitions
	# --------------------
	# !!!!! Warning !!!!!
	# This will clear the current boot record. Should only be used before any
	# disk partitions are created.
	# ***************************************************************************
	echo -e "o\nw\n" > $DT_TEMP_FILE
	fdisk $DT_HD < $DT_TEMP_FILE > /dev/null 2>&1
	udevadm settle
	sync
	# Grub bootloader is always installed on the boot partition.
	# to load grub we need to have another boot in the mbr loader
	# which is what the line below does.
	dd if=$DT_DRMK_PATH/mbr of=$DT_HD
	
	if [ "$DT_PARTN_UP" = "ON" ]; then
		$DT_SCRIPTS/upinit.sh --disk=$DT_HD --size=$DT_UP_SIZE --file=$DT_UP_IMAGE
		if [ $? -eq 0 ]; then
			DT_PART_NUM=2
		else
			echo "Could not create Dell Utility Partition."
			exit 1
		fi;
	fi
	if [ "$DT_PARTN_DEP" = "ON" ]; then
		createdp
	fi
else
	echo 'Creating GPT (UEFI mode) partitions...'

	# Utiilty partition is not supported for UEFI mode deployments
	[ -z "$DT_PARTN_UP" ] && DT_PARTN_UP=OFF

	# ***************************************************************************
	# Clear all partitions
	# --------------------
	# !!!!! Warning !!!!!
	# This will clear the current boot record. Should only be used before any
	# disk partitions are created.
	# ***************************************************************************
	echo -e "mklabel gpt\nquit\n" > $DT_TEMP_FILE
	parted $DT_HD < $DT_TEMP_FILE > /dev/null 2>&1
	udevadm settle
	sync
	# Grub2 bootloader is installed on the /boot/efi (EFI SYSTEM PARTITION) partition.
	# to load grub2 we need to have another boot in the mbr loader
	# which is what the line below does.
	dd if=$DT_DRMK_PATH/mbr of=$DT_HD 
	
	if [ "$DT_PARTN_DEP" = "ON" ]; then
		createdp_gpt
	fi
fi
