#!/bin/bash
# ***************************************************************************
# *                         WARRANTY DISCLAIMER
# ***************************************************************************
# * THE SAMPLE SCRIPT IS BEING PROVIDED TO YOU "AS IS".  DELL DISCLAIMS ANY
# * AND ALL WARRANTIES, EXPRESS, IMPLIED OR STATUTORY, WITH RESPECT TO THE
# * SAMPLE SCRIPT, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND ANY WARRANTY
# * OF NON-INFRINGEMENT. YOU WILL USE THE SAMPLE SCRIPT AT YOUR OWN RISK.
# * DELL SHALL NOT BE LIABLE TO YOU FOR ANY DIRECT OR INDIRECT DAMAGES
# * INCURRED IN USING THE SAMPLE SCRIPT. IN NO EVENT SHALL DELL OR ITS
# * SUPPLIERS BE RESPONSIBLE FOR ANY DIRECT OR INDIRECT DAMAGES WHATSOEVER
# * (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF PROFITS, LOSS OF USE,
# * LOSS OF DATA, BUSINESS INTERRUPTION, OR OTHER PECUNIARY LOSS, NOR FOR
# * PUNITIVE, INCIDENTAL, CONSEQUENTIAL, OR SPECIAL DAMAGES OF ANY KIND,
# * UNDER ANY PART OF THIS AGREEMENT, EVEN IF ADVISED OR AWARE OF THE
# * POSSIBILITY OF SUCH DAMAGES).
# ***************************************************************************

# ***************************************************************************
# * Script Name: racrep.sh
# *
# * Purpose:
# *   This script configures/replicates Remote Access Controller options
# *
# * Layout: 
# *   This script is divided into the following sections for ease of use and
# *   extensibility:
# *     Section 1: Set Paths for Deployment Toolkit Tools and Scripts
# *     Section 2: Ask User's Input, Input Verification, and Code Execution
# *
# * Tools Used: 
# *   racadm to configure Dell Remote Access Controller
# *
# * External Dependencies
# *   DT_TOOLS, DT_SYSTEMS set by tkenvset.sh let task scripts
# *   know where they can find Toolkit Tools and Configuration scripts.
# *   DT_PLATFORM is set by tkenvset.sh using Platform Discovery
# *
# * Input Parameters
# *	  $1 This is the filename containing the captured Settings.
# *	  $2 This is the ip to be assigend to this RAC.
# *	     This is required because IP Addresses will not be replicated.
# *	
# *	Note: These two parameters can be set in this file manually as well.
# *	
# ***************************************************************************

# Print usage if script is not called with correct number of arguments
noarg ()
{
        echo "usage: racrep <filename>"
	echo Filename is the name of the file to capture the Settings into.
        exit 2
}

# ***************************************************************************
# * Function to display the outcome (Success or Failure and Error Handling)
# * Expects $? as an argument to display the Return/Error Code.
# ***************************************************************************
toolfail ()
{
	echo racadm failed with error code $1.
	$DT_SCRIPTS/errhndl.sh $0 32
	# Exit the script if Return Value is 2
	if [ $? -eq 2 ]; then
		echo "Exiting."
		exit
	fi
}

syscfg_toolfail ()
{
	echo syscfg failed with error code $1.
	$DT_SCRIPTS/errhndl.sh $0 32
	# Exit the script if Return Value is 2
	if [ $? -eq 2 ]; then
		echo "Exiting."
		exit
	fi
}


# **************************************************************************
# * Functions checks the version of remote access controller on the server
# * sets the variable racversion=3/4/5.
# **************************************************************************
check_rac_ver()
{
    rac3=1028:0008
	rac4=1028:0012
    test=""
	test=`lspci -i $DT_TOOLS/pci.ids -d $rac3`
	if [ -n "$test" ]; then
		racversion=3
	fi
    test=""
	test=`lspci -i $DT_TOOLS/pci.ids -d $rac4`
	if [ -n "$test" ]; then
		racversion=4
	fi
    test=""
	test=`syscfg | grep racreset`
	if [ -n "$test" ]; then
		racversion=5
	fi
}

do_rac5_rep ()
{

	#Ignore the the DT_NICCFG for rac5
	#we have already commented it in the config file
	#generated by raccap.sh
	$DT_TOOLS/syscfg -i $DT_RAC_CAPFILE
	retval=$?
	if [ $retval -ne 0 ]; then
		syscfg_toolfail $retval
	fi
}


# ***************************************************************************
# * Section 1: Set Paths for Deployment Toolkit Tools and Scripts
# ***************************************************************************
#check for proper usage of the script
if [ -z $1 ]; then
        noarg
fi

if [ -z $DT_TOOLS ]; then
	. tkenvset.sh
fi

# #############################################################################
#                          USER MODIFICATION REQUIRED
# Below are the user defined variables that this script uses.
# In a full deployment, these variables should be customized to fit the 
# user's needs.
# #############################################################################
[ -z "$DT_RAC_CAPFILE" ] &&	DT_RAC_CAPFILE=$DT_SYSTEMS/$DT_PLATFORM/raccfg.ini

[ -z "$DT_NICCFG" ] && DT_NICCFG=10.98.8.121

# ***************************************************************************
# * Section 2: Ask User's Input, Input Verification, and Code Execution
# ***************************************************************************
# 1st parameter is the configuration file name
if [ ! -z $1 ]; then 
	DT_RAC_CAPFILE=$1
fi

# 2nd parameter is the IP to be assigned to the RAC.
if [ ! -z $2 ]; then
	DT_NICCFG=$2
fi

# Input Verification
if [ -z $DT_RAC_CAPFILE ]; then
	$DT_SCRIPTS/errhndl.sh $0 33
	exit
fi;

racversion=0
check_rac_ver

if [ $racversion -eq 0 ]; then
	#Nothing to do if we find no RAC on the system.
	$DT_SCRIPTS/errhndl.sh $0 34
	exit 0;
fi

if [ $racversion -eq 5 ]; then
	do_rac5_rep
	exit $retval
fi

if [ "$DT_NICCFG" = "" ]; then
	$DT_SCRIPTS/errhndl.sh $0 30
else
	$DT_TOOLS/racadm setniccfg -s $DT_NICCFG
	retval=$?
	if [ $retval -ne 0 ]; then 
		toolfail $retval
	fi;

	$DT_TOOLS/racadm racreset
	retval=$?
	if [ $retval -ne 0 ]; then
		toolfail $retval
	fi;
fi;

if [ ! -e $DT_RAC_CAPFILE ]; then
	$DT_SCRIPTS/errhndl.sh $0 58
	exit
fi;

echo Waiting for RAC to reset.
el=255
while (( $el != 0 )); do
        sleep 5
        racadm getsysinfo > /dev/null 2>&1
        el=$?
done

echo Remote Access Controller configured with $DT_NICCFG
echo NOTE: If the configuration file contains an IP address for the
echo Remote Access Controller then the Remote Access Controller will be
echo configured with the IP address from the configuration file.
$DT_TOOLS/racadm config -f $DT_RAC_CAPFILE
retval=$?

if [ $retval -ne 0 ]; then
	toolfail $retval
fi;

echo Remote Access Controller configured with $DT_RAC_CAPFILE
