#!/bin/bash

# ***************************************************************************
# *                         WARRANTY DISCLAIMER
# ***************************************************************************
# * THE SAMPLE SCRIPT IS BEING PROVIDED TO YOU "AS IS".  DELL DISCLAIMS ANY
# * AND ALL WARRANTIES, EXPRESS, IMPLIED OR STATUTORY, WITH RESPECT TO THE
# * SAMPLE SCRIPT, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND ANY WARRANTY
# * OF NON-INFRINGEMENT. YOU WILL USE THE SAMPLE SCRIPT AT YOUR OWN RISK.
# * DELL SHALL NOT BE LIABLE TO YOU FOR ANY DIRECT OR INDIRECT DAMAGES
# * INCURRED IN USING THE SAMPLE SCRIPT. IN NO EVENT SHALL DELL OR ITS
# * SUPPLIERS BE RESPONSIBLE FOR ANY DIRECT OR INDIRECT DAMAGES WHATSOEVER
# * (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF PROFITS, LOSS OF USE,
# * LOSS OF DATA, BUSINESS INTERRUPTION, OR OTHER PECUNIARY LOSS, NOR FOR
# * PUNITIVE, INCIDENTAL, CONSEQUENTIAL, OR SPECIAL DAMAGES OF ANY KIND,
# * UNDER ANY PART OF THIS AGREEMENT, EVEN IF ADVISED OR AWARE OF THE
# * POSSIBILITY OF SUCH DAMAGES).
# ***************************************************************************
# (c) Dell Inc, 2007
# ***************************************************************************
# * Script Name: raidcfg.sh
# *
# * Purpose: 
# *   This script configures the RAID controllers detected on the system.
# *
# * Layout: 
# *   This script is divided into the following sections for ease of use and
# *   extensibility:
# *     Section 1: Set Paths for Deployment Toolkit Tools and Scripts
# *     Section 2: Ask User's Input, Input Verification, and Code Execution
# *
# * Tools Used: raidcfg - RAID tool to configure the RAID controller.
# *
# * External Dependencies
# *   DT_TOOLS set by tkenvset.sh let task scripts
# *   know where they can find Toolkit Tools and Configuration scripts.
# *
# * Input Parameters
# *	  $1 Is the filename to log the raidcfg tool messages into.
# *
# * GLOBAL WARNING
# *   This script updates the activity/error log and requires write access to
# *   the file defined by DT_RAID_LOGFILE
# *
# ***************************************************************************

# ***************************************************************************
# * Function to display the outcome (Success or Failure and Error Handling)
# * Expects $? as an argument to display the Return/Error Code.
# ***************************************************************************
toolfail ()
{
	echo raidcfg failed with error code $1.
	$DT_SCRIPTS/errhndl.sh $0 $errcode
	# Exit the script if Return Value is 2
	if [ $? -eq 2 ]; then
		echo "Exiting."
		exit
	fi
}

# ***************************************************************************
# Function to configure a controller.
# $1 is the Controller ID of the controller to be configured.
# ***************************************************************************
create ()
{
	errcode=55
	CONT_ID=$1

	# Get the array disk ids on the selected controller.
	echo "Getting free array disk information on controller $CONT_ID"
	ARRAY_DISKS=''
	eval `$DT_TOOLS/raidcfg -se -envn=ARRAY_DISKS -envc=getfreeadisks -c=$CONT_ID $DT_RAID_LOGFILE_OPT | grep ARRAY_DISKS=` > /dev/null 2>&1
	echo "Free Array Disk information: $ARRAY_DISKS"
	if [ -z $ARRAY_DISKS ]; then
		echo "Could not get Free Array Disk Information on Controller $CONT_ID"
		return
	fi
	echo

	RAID50_PARAM=
	RAID_CFG_SUCCESS=0

# Uncomment the following line to try and create RAID 50.
# NOTE: spanlength should be chosen depending on the number of hard disks
# connected to the controller. Refer DTK CLI guide for more details.
# RAID50_PARAM="spanlength=3"

# Use the following line instead of the next line to try and create RAID 60, RAID 50 and 10 as well
# for r in 60 50 10 5 1 0;
	for r in 6 5 1 0;
	do
		RAID_TYPE=$r
		echo Checking if RAID $RAID_TYPE can be created on controller $CONT_ID ...

		$DT_TOOLS/raidcfg controller action=createvdisk controllerid=$CONT_ID adisk=$ARRAY_DISKS raid=$RAID_TYPE $RAID50_PARAM $DT_RAID_LOGFILE_OPT > /dev/null
		retval=$?
		if [ $retval -eq 0 ]; then
			RAID_CFG_SUCCESS=1
			break
		fi
		RAID50_PARAM=
	done

	# Could not create a VD
	if [ $RAID_CFG_SUCCESS -ne 1 ]; then
		echo "Error: Could not configure RAID on controller $CONT_ID"
	else
		echo Successfully created RAID $RAID_TYPE on controller $CONT_ID
	fi;
}

# ***************************************************************************
# * Section 1: Set Paths for Deployment Toolkit Tools and Scripts
# ***************************************************************************
if [ -z $DT_TOOLS ]; then
	. tkenvset.sh
fi

# ***************************************************************************
# * Section 2: Ask User's Input, Input Verification, and Code Execution
# ***************************************************************************

# Path and filename where log file is to be created, for ex: /tmp/log.txt
# Note:  The path must exist and the media must be writeable.
[ ! -z $1 ] && DT_RAID_LOGFILE=$1

# Input Verification
if [ -z "$DT_RAID_LOGFILE" ]; then
	$DT_SCRIPTS/errhndl.sh $0 01
fi

DT_RAID_LOGFILE_OPT=
if [ ! -z "$DT_RAID_LOGFILE" ]; then
	DT_RAID_LOGFILE_OPT="-l=$DT_RAID_LOGFILE"
fi

# RAIDCFG tool needs to be executed from within the tools directory.
pushd $DT_TOOLS > /dev/null

# Get the list of controllers.
CONT_LIST=`$DT_TOOLS/raidcfg -se -envn=CONT_LIST -envc=getcontrollerslots $DT_RAID_LOGFILE_OPT | grep CONT_LIST= | cut -d = -f2` > /dev/null 2>&1
if [ -z "$CONT_LIST" ]; then
	echo "Error: Could not get RAID Controller Slot Information"
	if [ ! -z "$DT_RAID_LOGFILE" ]; then
		echo Details are logged in $DT_RAID_LOGFILE
	fi
	exit
fi

# Loop through the list of controllers and try to configure them.
for i in $CONT_LIST; do
	create $i
done

# Return to the original directory.
popd > /dev/null

if [ ! -z "$DT_RAID_LOGFILE" ]; then
	echo Details are logged in $DT_RAID_LOGFILE
fi;
