#!/bin/bash

# ***************************************************************************
# *                         WARRANTY DISCLAIMER
# ***************************************************************************
# * THIS SCRIPT IS BEING PROVIDED TO YOU "AS IS".  DELL DISCLAIMS ANY
# * AND ALL WARRANTIES, EXPRESS, IMPLIED OR STATUTORY, WITH RESPECT TO THE
# * SCRIPT, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND ANY WARRANTY
# * OF NON-INFRINGEMENT. YOU WILL USE THIS SCRIPT AT YOUR OWN RISK.
# * DELL SHALL NOT BE LIABLE TO YOU FOR ANY DIRECT OR INDIRECT DAMAGES
# * INCURRED IN USING THE SCRIPT. IN NO EVENT SHALL DELL OR ITS
# * SUPPLIERS BE RESPONSIBLE FOR ANY DIRECT OR INDIRECT DAMAGES WHATSOEVER
# * (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF PROFITS, LOSS OF USE,
# * LOSS OF DATA, BUSINESS INTERRUPTION, OR OTHER PECUNIARY LOSS, NOR FOR
# * PUNITIVE, INCIDENTAL, CONSEQUENTIAL, OR SPECIAL DAMAGES OF ANY KIND,
# * UNDER ANY PART OF THIS AGREEMENT, EVEN IF ADVISED OR AWARE OF THE
# * POSSIBILITY OF SUCH DAMAGES).
# ***************************************************************************

# !!! WARNING !!! Please do not Edit or Modify this file.

##############################################################################
# Default values.
##############################################################################

# Default behaviour - Do not overwrite an existing UP.
UP_OVERWRITE=FALSE

# Mount point of the new Dell Utility Partition
UP_MOUNT_POINT=/tmp/up

# Temp file to store fdisk commands.
UP_TEMP_FILE=/tmp/dtkfdisktemp

##############################################################################

##############################################################################
# Error Code definitions
##############################################################################
SUCCESS=0
DISK_ID_NOT_SPECIFIED=1
SIZE_NOT_SPECIFIED=2
FILE_NOT_SPECIFIED=3
PATH_NOT_SPECIFIED=4
NO_OVERWRITE=5
PARTITIONS_EXITS=6
DOS_FILES_NOT_FOUND=7
UP_IMAGE_NOT_FOUND=8
FORMAT_ERROR=9
INVALID_SIZE=10
SYSDOS_ERROR=11
SYSTEM_ERROR=12
INVALID_ARGUMENT=13
UNZIP_ERROR=14

INVALID_DISK=31
MOUNT_FAILED=32
MOUNT_POINT_ERROR=33
FDISK_ERROR=34

##############################################################################
# Error Messages
#
# Error Codes defined above index into the array "err" to get the correct
# error message
##############################################################################
err[$SUCCESS]="Dell Utility Partition created successfully."
err[$DISK_ID_NOT_SPECIFIED]="Error: Disk ID not specified."
err[$SIZE_NOT_SPECIFIED]="Error: SIZE not specified."
err[$FILE_NOT_SPECIFIED]="Error: Utility Partition Image file not specified."
err[$PATH_NOT_SPECIFIED]="Error: DRMK Path not specified."

err[$DOS_FILES_NOT_FOUND]="Error: DOS file(s) not found at the specified path."
err[$UP_IMAGE_NOT_FOUND]="Error: UP IMAGE not found at the specified location."

err[$UNZIP_ERROR]="Error: unzip failed."
err[$FORMAT_ERROR]="Error: format failed"
err[$SYSDOS_ERROR]="Error: sysdrmk failed"
err[$SYSTEM_ERROR]="System Error"
err[$MOUNT_FAILED]="Error: Mount failed"
err[$FDISK_ERROR]="Error: fdisk failed"

err[$NO_OVERWRITE]="Error: Dell Utility Partition already exists. Please use --overwrite option to upgrade or overwrite."
err[$PARTITIONS_EXITS]="Error: Partitions exist. Please clear all the partitions before running upinit.sh"
err[$INVALID_DISK]="Error: Invalid disk specified"
err[$MOUNT_POINT_ERROR]="Cannot create mount point $UP_MOUNT_POINT. File exists."
err[$INVALID_SIZE]="Error: Invalid size. Size should be a number between 32 and 2000."

##############################################################################
# Valid options.
##############################################################################
opts[0]="--disk:-d:2:UP_DISK=\$var2:$DISK_ID_NOT_SPECIFIED"
opts[1]="--size:-s:2:UP_SIZE=\$var2:$SIZE_NOT_SPECIFIED"
opts[2]="--file:-f:2:UP_IMAGE=\$var2:$FILE_NOT_SPECIFIED"
opts[3]="--help:-h:1:usage $SUCCESS"
opts[4]="--overwrite:-o:1:UP_OVERWRITE=TRUE"

##############################################################################
# Function : usage
#
# Display the usage instruction/menu to the user.
##############################################################################
usage()
{
cat <<_EOF

Usage: 

    upinit.sh --disk|-d=disk --size|-s=size --file|-f=file
              [--overwrite|-o] [--help|-h]

      --help or -h prints help.
      --disk or -d disk device on which to create the Dell Utility Partition.
      --size or -s Size in MB for the Dell Utility Partition.
      --file or -f filename of Dell Utility Partition Image File.
 --overwrite or -o Installs the Dell Utility partition over an existing one.

Example:

	upinit --disk=/dev/sda --size=32 --file=upimg.bin

	In the example, upinit creates a Dell Utility Partition of size 32 MB on /dev/sda using upimg.bin file.

    Note: --overwrite should be used for upgrading/downgrading purposes.
          --size is not required when using --overwrite option because
          --overwrite does not resize an existing Dell Utility Partition.
          If any partitions exist they will not be deleted.
_EOF

exit $1
}

##############################################################################
# Function : usageerror
#
# Displays an error message, prints the usage menu and exits with an error code
#
# This function expects two arguments.
# $1 is the Exit Code. (See the section where the Error codes are defined)
##############################################################################
usageerror ()
{
	echo ${err[$1]}
	usage $1
}

##############################################################################
# Function : finish
#
# Simply display an Error message and exit with an appropriate error code.
##############################################################################
finish ()
{
	echo ${err[$1]}
	exit $1
}

##############################################################################
# Function : formatandpatch
# 
# Formats the created Utility Partition, Changes the partition type to 'de'
# and patches the Bootsector and MBR.
##############################################################################
formatandpatch ()
{
	udevadm settle
	sync
	mkfs.msdos -F 16 "$UP_DISK"1
	if [ $? -ne 0 ]; then
		finish $FORMAT_ERROR
	fi
	udevadm settle
	sync
	# Change the partition type to Dell Utility (de)
	numpart=`fdisk -l $UP_DISK | grep "$UP_DISK" | wc -l`
	if [ $numpart -eq 2 ]; then
		echo -e "t\nde\nw\n" > $UP_TEMP_FILE
	else
		echo -e "t\n1\nde\nw\n" > $UP_TEMP_FILE
	fi
	fdisk $UP_DISK < $UP_TEMP_FILE > /dev/null 2>&1
	if [ $? -ne 0 ]; then
		finish $FDISK_ERROR
	fi
	udevadm settle
	sync
	# Patch the boot sector
	$DT_TOOLS/sysdrmk $UP_DISK
	if [ $? -ne 0 ]; then
		finish $SYSDOS_ERROR
	fi

	# Patch the MBR - Only when we are creating a new Dell Utility Partition
	if [ "$UP_OVERWRITE" = "TRUE" ]; then
		return
	fi;
	echo "Patching MBR ..."
	dd if=$DT_DRMK_PATH/mbr of="$UP_DISK"
	if [ $? -ne 0 ]; then
		finish $SYSTEM_ERROR
	fi
}

##############################################################################
# Function : copyupfiles
# 
# Copies the files required for booting into DR-DOS and unzips the contents of
# Utility Partition Image file.
##############################################################################
copyupfiles ()
{
	# Mount the Utility Partition
	mount "$UP_DISK"1 $UP_MOUNT_POINT >/dev/null 2>&1
	if [ $? -ne 0 ]; then
		finish $MOUNT_FAILED
	fi

	# Extract the Dell Utility Partition Image
	unzip -d $UP_MOUNT_POINT $UP_IMAGE > /dev/null
	if [ $? -ne 0 ]; then
		umount $UP_MOUNT_POINT
		finish $UNZIP_ERROR
	fi

	umount $UP_MOUNT_POINT
	sync
}

##############################################################################
# Function : createpart
# 
# Creates a primary partition of type DE and sets the Bootable flag.
##############################################################################
createpart ()
{
	if [ -z $UP_SIZE ]; then
		usageerror $SIZE_NOT_SPECIFIED
	fi

	[ ! -z `echo $UP_SIZE | grep [^0-9]` ] || [ $UP_SIZE -lt 32 -o $UP_SIZE -gt 2000 ] && finish $INVALID_SIZE

	echo "Creating a Dell Utility Partition on $UP_DISK"

	# Create the partition
	echo -e "n\np\n1\n\n+${UP_SIZE}M\nw\n" > $UP_TEMP_FILE
	fdisk -S 63 -H 255 $UP_DISK < $UP_TEMP_FILE > /dev/null 2>&1
	if [ $? -ne 0 ]; then
		finish $FDISK_ERROR
	fi
	udevadm settle
	sync
}

##############################################################################
# Function : parseargs
#
# Used to parse the command line arguments passed by the user.
##############################################################################
parseargs ()
{
	while [ "X$1" != "X" ]; do
		validarg=0
		var1=`echo $1 | awk -F= '{print $1}'`
		var2=`echo $1 | awk -F= '{print $2}'`

		for (( i=0; i<5; i++ )); do
			lopt=`echo ${opts[$i]} | cut -d : -f 1`
			sopt=`echo ${opts[$i]} | cut -d : -f 2`

			if [ "$var1" = "$lopt" -o "$var1" = "$sopt" ]; then
				numvar=`echo ${opts[$i]} | cut -d : -f 3`
				if [ $numvar -eq 2 ]; then
					if [ -z $var2 ]; then
						errormessage=`echo ${opts[$i]} | cut -d : -f 5`
						usageerror $errormessage
						exit
					fi;
				fi;
				validarg=1
				command=`echo ${opts[$i]} | cut -d : -f 4`
				eval $command
			fi;
		done

		if [ $validarg -ne 1 ]; then
			err[$INVALID_ARGUMENT]="Error: Invalid Argument: $1"
			usageerror $INVALID_ARGUMENT
		fi;

		shift
	done
}

##############################################################################
# Section 1: Parse the command line arguments and set all the required variables.
##############################################################################
cat <<_EOF
upinit version 1.0
Copyright (c) 2002-2012 Dell Inc.

This utility creates a utility partition for your system.
This utility requires a Utility Partition image file (upimg.bin) which can be
found under /opt/dell/toolkit/systems folder in the Deployment Toolkit ISO.  For more information, please consult the Deployment Toolkit Documentation.
_EOF

if [ $# -eq 0 ]; then
	usage $SUCCESS
fi

parseargs $*

##############################################################################
# Section 2: Set The environment variables if required.
##############################################################################
if [ -z $DT_TOOLS ]; then
	. tkenvset.sh
fi
 
##############################################################################
# Check if we have everthing we need to create a Dell Utility Partition
##############################################################################
if [ -z $UP_DISK ]; then
	usageerror $DISK_ID_NOT_SPECIFIED
#elif [ -z $UP_SIZE ]; then
#	usageerror $SIZE_NOT_SPECIFIED 
elif [ -z $UP_IMAGE ]; then
	usageerror $FILE_NOT_SPECIFIED
elif [ ! -e $UP_IMAGE ]; then
	finish $UP_IMAGE_NOT_FOUND
elif [ ! -f $UP_IMAGE ]; then
	usageerror $FILE_NOT_SPECIFIED
elif [ -z $DT_DRMK_PATH ]; then
	finish $PATH_NOT_SPECIFIED
fi

# Check for DRMK files
if [ ! -e $DT_DRMK_PATH/mbr ]; then
	finish $DOS_FILES_NOT_FOUND
fi

##############################################################################
# Done with checking the arguments. Now try to create a Dell Utility Partition
##############################################################################

if [ ! -e $UP_MOUNT_POINT ]; then
	mkdir $UP_MOUNT_POINT
else
	if [ ! -d $UP_MOUNT_POINT ]; then
		finish $MOUNT_POINT_ERROR
	fi;
fi;


# Detect if there are any partitions on the disk.
numpart=`fdisk -l $UP_DISK | grep "$UP_DISK" | wc -l`
if [ $numpart -eq 0 ]; then
	finish $INVALID_DISK
elif [ $numpart -eq 1 ]; then
	# There are no partitions on this disk.
	UP_OVERWRITE=FALSE
	createpart
	formatandpatch
	copyupfiles
elif [ $numpart -gt 1 ]; then
	# Check if there is a Dell Utility Partition.
	tempvar=`fdisk -l $UP_DISK | grep "Dell Utility" | cut -d ' ' -f1`
	if [ "$tempvar" = "${UP_DISK}1" ]; then
		# Yes. This one is a Dell Utility Partition.
		if [ "$UP_OVERWRITE" = "TRUE" ]; then
			# User wants us to overwrite.
			formatandpatch
			copyupfiles
		else
			# User does not want to overwrite.
			finish $NO_OVERWRITE
		fi
	else
		finish $PARTITIONS_EXITS
	fi
else
	# More than one partition exists. Just exit the script.
	finish $PARTITIONS_EXITS
fi

finish $SUCCESS
